@@ -1,18 +1,24 @@
 from __future__ import annotations
 
-import re
-from dataclasses import dataclass, field
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 
+import re
 import yaml
 from commonmodel.field_types import FieldType, FieldTypeLike, ensure_field_type
+from commonmodel.utils import FrozenPydanticBase
 
 
-@dataclass(frozen=True)
-class Field:
+# TODO: validator support (NotNull Immutable Min Max ...?? see reference)
+class Validator(FrozenPydanticBase):
+    name: str
+    value: Optional[str] = None
+    parameters: Optional[Dict[str, Any]] = None
+
+
+class Field(FrozenPydanticBase):
     name: str
     field_type: FieldType
-    validators: List[Validator] = field(default_factory=list)
+    validators: List[Validator] = []
     description: Optional[str] = None
 
     def is_nullable(self) -> bool:
@@ -22,27 +28,17 @@ class Field:
         return True
 
 
-# TODO: validator support (NotNull Immutable Min Max ...?? see reference)
-@dataclass(frozen=True)
-class Validator:
-    name: str
-    value: Optional[str] = None
-    parameters: Optional[Dict[str, Any]] = None
-
-
 SchemaKey = str
 SchemaName = str
 
 
-@dataclass(frozen=True)
-class Relation:
+class Relation(FrozenPydanticBase):
     name: str
     schema_key: SchemaKey
     fields: Dict[str, str]
 
 
-@dataclass(frozen=True)
-class Implementation:
+class Implementation(FrozenPydanticBase):
     schema_key: SchemaKey
     fields: Dict[str, str]
 
@@ -56,27 +52,26 @@ class Implementation:
         )
 
 
-@dataclass(frozen=True)
-class FieldRoles:
-    measures: List[str] = field(default_factory=list)
-    dimensions: List[str] = field(default_factory=list)
-    primary_measure: Optional[str] = None
-    primary_dimension: Optional[str] = None
-    creation_ordering: List[str] = field(default_factory=list)
-    modification_ordering: List[str] = field(default_factory=list)
+class FieldRoles(FrozenPydanticBase):
+    primary_identifier: Optional[str] = None
+    measures: List[str] = []
+    dimensions: List[str] = []
+    # primary_measure: Optional[str] = None  # TODO: or primary_dimensionS plural? or just use ordering of measures?
+    # primary_dimension: Optional[str] = None
+    creation_ordering: List[str] = []
+    modification_ordering: List[str] = []
 
 
-@dataclass(frozen=True)
-class Schema:
+class Schema(FrozenPydanticBase):
     name: str
     namespace: Optional[str]
     version: Optional[str]
     description: str
     unique_on: List[str]
     fields: List[Field]
-    field_roles: FieldRoles = field(default_factory=FieldRoles)
-    relations: List[Relation] = field(default_factory=list)
-    implementations: List[Implementation] = field(default_factory=list)
+    field_roles: FieldRoles = FieldRoles()
+    relations: List[Relation] = []
+    implementations: List[Implementation] = []
     immutable: bool = False
     raw_definition: Optional[str] = None
     # extends: Optional[
@@ -133,16 +128,10 @@ class Schema:
 SchemaLike = Union[Schema, SchemaKey, SchemaName]
 
 
-class SchemaTranslation:
-    def __init__(
-        self,
-        translation: Optional[Dict[str, str]] = None,
-        from_schema_key: Optional[SchemaKey] = None,
-        to_schema_key: Optional[SchemaKey] = None,
-    ):
-        self.translation = translation
-        self.from_schema_key = from_schema_key
-        self.to_schema_key = to_schema_key
+class SchemaTranslation(FrozenPydanticBase):
+    translation: Optional[Dict[str, str]] = None
+    from_schema_key: Optional[SchemaKey] = None
+    to_schema_key: Optional[SchemaKey] = None
 
     def as_dict(self) -> Dict[str, str]:
         if not self.translation:
@@ -175,6 +164,12 @@ def schema_from_yaml(yml: str, **overrides: Any) -> Schema:
     return schema_from_dict(yaml.load(yml, Loader=yaml.SafeLoader), **overrides)
 
 
+def schema_from_yaml_file(pth: str, **overrides: Any) -> Schema:
+    with open(pth) as f:
+        s = f.read()
+    return schema_from_yaml(s, **overrides)
+
+
 def schema_from_dict(d: Dict[str, Any], **overrides: Any) -> Schema:
     d = clean_raw_schema_defintion(d)
     return build_schema_from_dict(d, **overrides)
@@ -184,6 +179,19 @@ def clean_keys(d: dict) -> dict:
     return {k.lower().replace(" ", "_"): v for k, v in d.items()}
 
 
+re_field_expr = re.compile(r"\w+(\([^)]+\))?")
+
+
+def process_field_string_to_dict(f: str) -> Dict:
+    d = {"validators": []}
+    for i, m in enumerate(re_field_expr.finditer(f)):
+        if i == 0:
+            d["type"] = m.group(0)
+        else:
+            d["validators"].append(m.group(0))
+    return d
+
+
 def clean_raw_schema_defintion(raw_def: dict) -> dict:
     """"""
     raw_def = clean_keys(raw_def)
@@ -191,7 +199,7 @@ def clean_raw_schema_defintion(raw_def: dict) -> dict:
     raw_def["fields"] = []
     for name, f in raw_fields.items():
         if isinstance(f, str):
-            f = {"type": f}
+            f = process_field_string_to_dict(f)
         nf = {"name": name, "field_type": f.pop("type", None)}
         nf.update(f)
         raw_def["fields"].append(nf)
